@@ -1,5 +1,8 @@
+from jinja2 import Environment, PackageLoader
+
 import base64
 import click
+import commentjson
 import getpass
 import io
 import json
@@ -13,6 +16,7 @@ import requests
 import sys
 import textwrap
 import uuid
+import binascii
 import zipfile
 
 @click.group()
@@ -118,7 +122,8 @@ def list_projects(query):
 @cli.command(name='grow')
 @click.argument('project')
 @click.option('path', '--dir', '-d', type=click.Path())
-def generate_project(project, path):
+@click.option('--vscode', is_flag=True)
+def generate_project(project, path, vscode):
     """ Expand a new or existing project """
     if path:
         splash('grow project in {}'.format(path))
@@ -133,22 +138,23 @@ def generate_project(project, path):
     click.echo('[OK]')
     
     with load_session() as session:
+        click.echo('fetching metadata...\t', nl=False)
+        r = session.get('{}/acorn/create/context/project?project_id={}'.format(ACORN, projectid))
+        if not r.ok:
+            sys.exit('[FAIL] unable to download metadata')
+        click.echo('[OK]')
+
+        metadata = r.json()
         if path:
             click.echo('checking directory...\t', nl=False)
-            target = path
+            target = os.path.abspath(path)
             os.makedirs(target, exist_ok=True)
             if not os.path.isdir(target):
                 sys.exit('[FAIL] "{}" does not exist or is not a directory.'.format(target))
             click.echo('[OK]')
         else:
-            click.echo('fetching metadata...\t', nl=False)
-            r = session.get('{}/acorn/create/context/project?project_id={}'.format(ACORN, projectid))
-            if not r.ok:
-                sys.exit('[FAIL] unable to download metadata')
-            click.echo('[OK]')
-
             click.echo('checking directory...\t', nl=False)
-            target = os.path.join(os.getcwd(), r.json()['name'])
+            target = os.path.join(os.getcwd(), metadata['name'])
             try:
                 os.makedirs(target)
             except FileExistsError:
@@ -159,6 +165,11 @@ def generate_project(project, path):
         if os.path.isfile(manifest):
             click.echo('cleaning directory...\t', nl=False)
             do_clean(target, manifest)
+        
+        config = os.path.join(target, metadata['name'] + '.cfg')
+        if not os.path.exists(config):
+            with open(config, 'w') as f:
+                f.write(CONFIG.format(cwd=target, issuer=uuid.uuid1(), secret=token_hex(8)))
 
         click.echo('downloading project...\t', nl=False)
         r = session.get('{}/acorn/create/context/project/{}/view/development?branch=master'.format(ACORN, projectid))
@@ -214,6 +225,10 @@ def generate_project(project, path):
         if os.path.isdir(parent):
             with open(os.path.join(parent, '.gitignore'), 'w') as f:
                 f.writelines(files)
+
+    if vscode:
+        configure_vscode(metadata['name'], target)
+
     click.echo('[SUCCESS]')
 
 @cli.command(name='clean')
@@ -316,5 +331,56 @@ def splash(action):
     click.echo('action requested: {}'.format(action).center(80))
     click.echo('-'*80)
 
+def configure_vscode(project, path):
+    fqn = os.path.join(path, '.vscode', 'tasks.json')
+    updated = json.loads(
+        ENV.get_template('tasks.json.j2').render(project=project)
+    )
+
+    try:
+        with open(fqn) as f:
+            existing = commentjson.load(f)
+    except commentjson.JSONLibraryException:
+        sys.exit('[FAIL] unable to parse existing vscode tasks.')
+    except FileNotFoundError:
+        existing = {}
+    
+    if 'version' in existing and existing['version'] != updated['version']:
+        sys.exit('[FAIL] expecting vscode task version {} (is {}).'.format(
+            updated['version'],
+            existing['version']))
+    
+    overwritten = {x['label'] for x in updated['tasks']}
+    updated['tasks'].extend(filter(
+        lambda x: x.get('label', None) not in overwritten,
+        existing.get('tasks', ())))
+
+    overwritten = {x['id'] for x in updated['inputs']}
+    updated['inputs'].extend(filter(
+        lambda x: x.get('id', None) not in overwritten,
+        existing.get('inputs', ())))
+    
+    existing.update(updated)
+    os.makedirs(os.path.join(path, '.vscode'), exist_ok=True)
+    with open(fqn, 'w') as f:
+        commentjson.dump(existing, f, indent=4)
+
+def token_hex(nbytes: int = 16):
+    return binascii.hexlify(os.urandom(nbytes)).decode('ascii')
+
 WINDOWS = platform.system() == 'Windows'
-ACORN = 'https://acorn.squirreltechnologies.nz'
\ No newline at end of file
+ACORN = 'https://acorn.squirreltechnologies.nz'
+ENV = Environment(
+    loader=PackageLoader('pyacorn', 'templates')
+)
+CONFIG = '''SYSTEM_STORAGE_DSN=sqlite:///{cwd}/.database.db
+SYSTEM_STORAGE_ATTACHMENTS={cwd}/.attachments
+SYSTEM_AUTH_ISSUER={issuer}
+SYSTEM_AUTH_SECRET={secret}
+SYSTEM_NETWORK_CACHE=tcp://127.0.0.1:5555
+SYSTEM_NETWORK_SUBMIT=tcp://127.0.0.1:5556
+SYSTEM_NETWORK_NOTIFY=tcp://127.0.0.1:5557
+SYSTEM_NETWORK_CONTROL=tcp://127.0.0.1:5558
+SYSTEM_MQ_DATA={cwd}/.mq
+SYSTEM_MQ_SUBSCRIBE=http://127.0.0.1:5000
+'''
\ No newline at end of file
